@@ -276,6 +276,8 @@ IDocument::notifyFindStarted ()
 void
 IDocument::notifyLoad ()
 {
+    // Empty the cache to avoid displaying pages from previous file.
+    clearCache ();
     // Add the two first pages, if they exists, to the cache.
     addPageToCache (1);
     if ( 1 < getNumPages () )
@@ -722,7 +724,7 @@ const gchar *
 IDocument::getFormat ()
 {
     if ( NULL == m_Format )
-    {        
+    {
         return "";
     }
     return m_Format;
@@ -1365,7 +1367,7 @@ IDocument::addPageToCache (gint pageNum)
         job->setDocument (this);
         job->setPageNumber (pageNum);
         IJob::enqueue (job);
-        
+
         G_LOCK (pageSearch);
         // Check which cached page to drop.
         if ( g_list_length (m_PageCache) >= CACHE_SIZE )
@@ -1416,7 +1418,7 @@ IDocument::getCachedPage (gint pageNum)
 {
     G_LOCK (pageSearch);
     PageCache *cached = NULL;
-    for ( GList *page = g_list_first (m_PageCache) ; 
+    for ( GList *page = g_list_first (m_PageCache) ;
           NULL != page && NULL == cached ;
           page = g_list_next (page) )
     {
@@ -1442,7 +1444,7 @@ IDocument::refreshCache ()
 {
     gint pageCount = g_list_length (m_PageCache);
     guint32 minAge = G_MAXUINT32;
-    for ( GList *page = g_list_first (m_PageCache) ; 
+    for ( GList *page = g_list_first (m_PageCache) ;
           NULL != page ;
           page = g_list_next (page) )
     {
@@ -1475,16 +1477,21 @@ IDocument::refreshCache ()
 void
 IDocument::clearCache ()
 {
+    G_LOCK (pageSearch);
     // Delete all cached pages.
-    for ( GList *page = g_list_first (m_PageCache) ; 
+    for ( GList *page = g_list_first (m_PageCache) ;
           NULL != page ;
           page = g_list_next (page) )
     {
         PageCache *cachedPage = (PageCache *)page->data;
+        G_LOCK (pageImage);
         delete cachedPage->pageImage;
+        G_UNLOCK (pageImage);
         delete cachedPage;
     }
     g_list_free (m_PageCache);
+    m_PageCache = NULL;
+    G_UNLOCK (pageSearch);
 }
 
 ///
