@@ -115,12 +115,15 @@ namespace ePDFView
             /// @param lastFolder The last folder used to save a file. This is
             ///                   used to show this folder when the save dialog
             ///                   appears.
+            /// @param fileName The file name to set as the initial name to
+            ///                 the save dialog.
             ///
             /// @return A copy of the file name that the user will try to use
             ///         to save or NULL if the user cancelled the operation.
             ///         This string will be freed by the presenter.
             ///
-            virtual gchar *saveFileDialog (const gchar *lastFolder) = 0;
+            virtual gchar *saveFileDialog (const gchar *lastFolder,
+                                           const gchar *fileName) = 0;
 
             ///
             /// @brief Changes the sensitivity of the "Find" action.
@@ -354,6 +357,14 @@ namespace ePDFView
             ///
             virtual void setCursor (ViewCursor cursorType) = 0;
 
+            ///
+            /// @brief Sets the view to full screen.
+            ///
+            /// @param fullScreen @a TRUE if the view should go full screen,
+            ///                   @a FALSE otherwise.
+            ///
+            virtual void setFullScreen (gboolean fullScreen) = 0;
+
             ///
             /// @brief Sets the text for the number of pages.
             ///
@@ -574,10 +585,10 @@ namespace ePDFView
             ///
             /// @param pter The main presenter that will drive the view.
             ///
-            IMainView (MainPter *pter) 
-            { 
+            IMainView (MainPter *pter)
+            {
                 g_assert ( NULL != pter && "Tried to set a NULL presenter." );
-                m_Pter = pter; 
+                m_Pter = pter;
             }
 
             /// The view's presenter.
