@@ -52,6 +52,7 @@ MainPter::MainPter (IDocument *document)
     m_Document->attach (this);
     m_View = NULL;
     m_PagePter = NULL;
+    m_FindPter = NULL;
     m_PasswordTries = 3;
 #if defined (DEBUG)
     G_LOCK (fileLoaded);
@@ -284,7 +285,7 @@ MainPter::setView (IMainView *view)
     delete m_PagePter;
     m_PagePter = new PagePter (m_Document);
     m_PagePter->setView (*view);
-    
+
     delete m_View;
     m_View = view;
     // Now that the view is set, it's time to set the initial 
@@ -292,6 +293,27 @@ MainPter::setView (IMainView *view)
     setInitialState ();
 }
 
+///
+/// @brief An URL link in the about box has been activated.
+///
+/// @param link The URL of the link activated.
+///
+void
+MainPter::aboutBoxLinkActivated (const gchar *link)
+{
+    gchar *browserCommandLine =
+        Config::getConfig ().getExternalBrowserCommandLine ();
+    gchar *commandLine = g_strdup_printf (browserCommandLine, link);
+    g_free (browserCommandLine);
+
+    GError *error = NULL;
+    if ( !g_spawn_command_line_async (commandLine, &error) )
+    {
+        g_error_free (error);
+    }
+    g_free (commandLine);
+}
+
 ///
 /// @brief The "Find" action was activated.
 ///
@@ -305,13 +327,24 @@ MainPter::findActivated ()
     m_FindPter->setView (getView ().getFindView ());
 }
 
+///
+/// @brief The "Full Screen" action was activated.
+///
+/// @param show TRUE if Full Screen is active, FALSE otherwise.
+///
+void
+MainPter::fullScreenActivated (gboolean active)
+{
+    getView ().setFullScreen (active);
+}
+
 ///
 /// @brief The "Go To First Page" action was activated.
 ///
 void
 MainPter::goToFirstPageActivated ()
 {
-    g_assert (NULL != m_Document && 
+    g_assert (NULL != m_Document &&
               "Tried to go to the first page of a NULL document.");
 
     m_PagePter->setNextPageScroll (PAGE_SCROLL_START);
@@ -544,7 +577,9 @@ MainPter::saveFileActivated ()
     Config &config = Config::getConfig ();
     gchar *lastFolder = config.getSaveFileFolder ();
     IMainView &view = getView ();
-    gchar *fileName = view.saveFileDialog (lastFolder);
+    gchar *originalFileName = g_path_get_basename (m_Document->getFileName ());
+    gchar *fileName = view.saveFileDialog (lastFolder, originalFileName);
+    g_free (originalFileName);
     g_free (lastFolder);
     if ( NULL != fileName )
     {
