@@ -45,7 +45,9 @@ namespace ePDFView
             IMainView &getView (void);
             void setView (IMainView *view);
 
+            void aboutBoxLinkActivated (const gchar *link);
             void findActivated (void);
+            void fullScreenActivated (gboolean active);
             void goToFirstPageActivated (void);
             void goToLastPageActivated (void);
             void goToNextPageActivated (void);
