@@ -16,8 +16,10 @@
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 #include <config.h>
+#include <gdk/gdk.h>
 #include <time.h>
 #include <poppler.h>
+#include <unistd.h>
 #include "epdfview.h"
 
 using namespace ePDFView;
@@ -158,7 +160,7 @@ PDFDocument::findTextInPage (gint pageNum, const gchar *textToFind)
               match = g_list_next (match) )
         {
             PopplerRectangle *matchRect = (PopplerRectangle *)match->data;
-            DocumentRectangle *rect = 
+            DocumentRectangle *rect =
                 new DocumentRectangle (matchRect->x1,
                                        (height - matchRect->y2),
                                        matchRect->x2,
@@ -195,7 +197,7 @@ PDFDocument::isLoaded ()
 /// @return TRUE if the file could be opened, FALSE otherwise.
 ///
 gboolean
-PDFDocument::loadFile (const gchar *filename, const gchar *password, 
+PDFDocument::loadFile (const gchar *filename, const gchar *password,
                     GError **error)
 {
     g_assert (NULL != filename && "Tried to load a NULL file name");
@@ -209,6 +211,35 @@ PDFDocument::loadFile (const gchar *filename, const gchar *password,
     }
     // Try to open the PDF document.
     GError *loadError = NULL;
+    // In case the user specified to read the PDF file from the stdin
+    // (i.e., -), then we save the contents of the stdin to a temporary
+    // file and use the URI to this temporary file to load.
+    if ( g_ascii_strcasecmp ("-", filename) == 0 )
+    {
+        gchar *tmpFileName;
+        gint fd = g_file_open_tmp ("epdfviewXXXXXX", &tmpFileName, error);
+        if ( -1 == fd )
+        {
+            return FALSE;
+        }
+        while ( !feof (stdin) )
+        {
+            gchar inputLine[512];
+            size_t readBytes = fread (inputLine, sizeof (char),
+                                      sizeof (inputLine) / sizeof (char),
+                                      stdin);
+            write (fd, inputLine, readBytes);
+        }
+        close (fd);
+
+        g_free (filename_uri);
+        filename_uri = g_filename_to_uri (tmpFileName, NULL, error);
+        g_free (tmpFileName);
+        if ( NULL == filename_uri )
+        {
+            return FALSE;
+        }
+    }
     PopplerDocument *newDocument =
         poppler_document_new_from_file (filename_uri, password, &loadError);
     g_free (filename_uri);
@@ -221,7 +252,7 @@ PDFDocument::loadFile (const gchar *filename, const gchar *password,
         DocumentError errorCode = DocumentErrorNone;
         if ( POPPLER_ERROR == loadError->domain )
         {
-            errorCode = DocumentErrorEncrypted; 
+            errorCode = DocumentErrorEncrypted;
         }
         else
         {
@@ -230,13 +261,13 @@ PDFDocument::loadFile (const gchar *filename, const gchar *password,
             // non translated string.
             // Maybe I'm wrong (very probable) but that's a wrong way.
             // So I'm reading the error code from the error string...
-            sscanf (loadError->message, "Failed to load document (error %d)", 
+            sscanf (loadError->message, "Failed to load document (error %d)",
                     (gint *)&errorCode);
         }
         g_error_free (loadError);
         // Get our error message.
         gchar *errorMessage = IDocument::getErrorMessage (errorCode);
-        g_set_error (error, 
+        g_set_error (error,
                      EPDFVIEW_DOCUMENT_ERROR, errorCode,
                      _("Failed to load document '%s'.\n%s\n"),
                      filename, errorMessage);
@@ -260,6 +291,15 @@ PDFDocument::loadFile (const gchar *filename, const gchar *password,
     PopplerIndexIter *outline = poppler_index_iter_new (m_Document);
     m_Outline = new DocumentOutline ();
     setOutline (m_Outline, outline);
+    // XXX: I still don't know why, but it seems that if I don't request
+    // a page here, when rendering a page Glib tells me
+    // that cannot register existing type `PopplerPage'... Yes, this
+    // is a kludge, but I don't know enough to check it why it does that.
+    PopplerPage *page = poppler_document_get_page (m_Document, 0);
+    if ( NULL != page )
+    {
+        g_object_unref (G_OBJECT (page));
+    }
 
     return TRUE;
 }
@@ -309,9 +349,12 @@ PDFDocument::loadMetadata (void)
     if ( 0 < creationDate )
     {
         struct tm *tmpTime = localtime ((const time_t *)&creationDate);
-        gchar *date = g_strnfill (DATE_LENGTH + 1, 0);
-        strftime (date, DATE_LENGTH, "%Y-%m-%d %H:%M:%S", tmpTime);
-        setCreationDate (date);
+        if ( NULL != tmpTime )
+        {
+            gchar *date = g_strnfill (DATE_LENGTH + 1, 0);
+            strftime (date, DATE_LENGTH, "%Y-%m-%d %H:%M:%S", tmpTime);
+            setCreationDate (date);
+        }
     }
     else
     {
@@ -324,9 +367,12 @@ PDFDocument::loadMetadata (void)
     if ( 0 < modDate )
     {
         struct tm *tmpTime = localtime ((const time_t *)&modDate);
-        gchar *date = g_strnfill (DATE_LENGTH + 1, 0);
-        strftime (date, DATE_LENGTH, "%Y-%m-%d %H:%M:%S", tmpTime);
-        setModifiedDate (date);
+        if ( NULL != tmpTime )
+        {
+            gchar *date = g_strnfill (DATE_LENGTH + 1, 0);
+            strftime (date, DATE_LENGTH, "%Y-%m-%d %H:%M:%S", tmpTime);
+            setModifiedDate (date);
+        }
     }
     else
     {
@@ -401,7 +447,7 @@ PDFDocument::setOutline (DocumentOutline *outline,
     {
         do
         {
-            PopplerAction *action = 
+            PopplerAction *action =
                 poppler_index_iter_get_action (childrenList);
             if ( POPPLER_ACTION_GOTO_DEST == action->type )
             {
@@ -468,7 +514,7 @@ PDFDocument::getPageSizeForPage (gint pageNum, gdouble *width, gdouble *height)
         {
             poppler_page_get_size (page, &pageHeight, &pageWidth);
         }
-        else 
+        else
         {
             poppler_page_get_size (page, &pageWidth, &pageHeight);
         }
@@ -529,7 +575,6 @@ PDFDocument::outputPostscriptPage (guint pageNum)
     }
 }
 
-
 ///
 /// @brief Renders a document's page.
 ///
