@@ -25,7 +25,9 @@
 #include <Config.h>
 
 #include <DocumentRectangle.h>
-#include <DocumentLink.h>
+#include <IDocumentLink.h>
+#include <DocumentLinkGoto.h>
+#include <DocumentLinkUri.h>
 #include <DocumentOutline.h>
 #include <DocumentPage.h>
 #include <IDocumentObserver.h>
@@ -35,13 +37,25 @@
 #include <IJob.h>
 #include <JobFind.h>
 #include <JobLoad.h>
+#include <JobPrint.h>
 #include <JobRender.h>
+#include <JobSave.h>
+#if defined (HAVE_CUPS)
+#endif // HAVE_CUPS
 
 #include <IFindView.h>
 #include <IPageView.h>
+#include <IPreferencesView.h>
+#if defined (HAVE_CUPS)
+#include <IPrintView.h>
+#endif // HAVE_CUPS
 #include <IMainView.h>
 #include <FindPter.h>
 #include <PagePter.h>
+#include <PreferencesPter.h>
+#if defined (HAVE_CUPS)
+#include <PrintPter.h>
+#endif // HAVE_CUPS
 #include <MainPter.h>
 
 #endif //!__E_PDF_VIEW_H__
