@@ -49,7 +49,10 @@ enum indexColumns
 
 // Forward declarations.
 static void main_window_about_box_cb (GtkWidget *, gpointer);
+static void main_window_about_box_url_hook (GtkAboutDialog *, const gchar *,
+                                            gpointer);
 static void main_window_find_cb (GtkWidget *, gpointer);
+static void main_window_fullscreen_cb (GtkToggleAction *, gpointer);
 static gboolean main_window_moved_or_resized_cb (GtkWidget *,
                                                  GdkEventConfigure *, gpointer);
 static void main_window_go_to_first_page_cb (GtkWidget *, gpointer);
@@ -79,7 +82,7 @@ static void main_window_print_cb (GtkWidget *, gpointer);
 #endif // HAVE_CUPS
 
 // The actions for menus and toolbars.
-static const GtkActionEntry g_NormalEntries[] = 
+static const GtkActionEntry g_NormalEntries[] =
 {
     { "FileMenu", NULL, N_("_File"), NULL, NULL, NULL },
     { "EditMenu", NULL, N_("_Edit"), NULL, NULL, NULL },
@@ -160,12 +163,18 @@ static const GtkActionEntry g_NormalEntries[] =
 
 static GtkToggleActionEntry g_ToggleEntries[] =
 {
+    { "FullScreen", NULL, N_("F_ull screen"), "F11",
+      N_("Toggle full screen window"),
+      G_CALLBACK (main_window_fullscreen_cb), FALSE },
+
     { "ShowToolBar", NULL, N_("Show _Toolbar"), NULL,
       N_("Show or hide the toolbar"),
       G_CALLBACK (main_window_show_toolbar_cb), TRUE },
+
     { "ShowStatusBar", NULL, N_("Show _Statusbar"), NULL,
       N_("Show or hide the statusbar"),
       G_CALLBACK (main_window_show_statusbar_cb), TRUE },
+
     { "ShowIndex", NULL, N_("Show I_ndex"), "F9",
       N_("Show or hide the document's outline"),
       G_CALLBACK (main_window_show_index_cb), FALSE },
@@ -288,6 +297,8 @@ MainView::openFileDialog (const gchar *lastFolder)
         gtk_file_filter_set_name (pdfFilter, 
                                   _("Portable Document Format (PDF) Files"));
         gtk_file_filter_add_mime_type (pdfFilter, "application/pdf");
+        gtk_file_filter_add_pattern (pdfFilter, "*.pdf");
+        gtk_file_filter_add_pattern (pdfFilter, "*.PDF");
 
         gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (openDialog), pdfFilter);
         // Set this filter as the default.
@@ -370,13 +381,13 @@ MainView::promptPasswordDialog ()
 }
 
 gchar *
-MainView::saveFileDialog (const gchar *lastFolder)
+MainView::saveFileDialog (const gchar *lastFolder, const gchar *fileName)
 {
     GtkWidget *saveDialog = gtk_file_chooser_dialog_new (_("Save PDF File"),
             GTK_WINDOW (m_MainWindow),
             GTK_FILE_CHOOSER_ACTION_SAVE,
             GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-            GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
+            GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
             NULL);
     gtk_dialog_set_alternative_button_order (GTK_DIALOG (saveDialog),
             GTK_RESPONSE_ACCEPT, GTK_RESPONSE_CANCEL, -1);
@@ -387,6 +398,12 @@ MainView::saveFileDialog (const gchar *lastFolder)
         gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (saveDialog),
                                              lastFolder);
     }
+    // Set the original file name to use as initial save name, if any.
+    if ( NULL != fileName )
+    {
+        gtk_file_chooser_set_current_name (GTK_FILE_CHOOSER (saveDialog),
+                                           fileName);
+    }
 
     // Add the file type filters.
     {
@@ -394,6 +411,8 @@ MainView::saveFileDialog (const gchar *lastFolder)
         gtk_file_filter_set_name (pdfFilter, 
                                   _("Portable Document Format (PDF) Files"));
         gtk_file_filter_add_mime_type (pdfFilter, "application/pdf");
+        gtk_file_filter_add_pattern (pdfFilter, "*.pdf");
+        gtk_file_filter_add_pattern (pdfFilter, "*.PDF");
 
         gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (saveDialog), pdfFilter);
         // Set this filter as the default.
@@ -422,7 +441,7 @@ MainView::saveFileDialog (const gchar *lastFolder)
 void
 MainView::sensitiveFind (gboolean sensitive)
 {
-    GtkAction *find = 
+    GtkAction *find =
         gtk_ui_manager_get_action (m_UIManager, "/MenuBar/EditMenu/Find");
     gtk_action_set_sensitive (find, sensitive);
 }
@@ -483,18 +502,18 @@ MainView::sensitivePrint (gboolean sensitive)
 }
 #endif // HAVE_CUPS
 
-void 
+void
 MainView::sensitiveReload (gboolean sensitive)
 {
-    GtkAction *reload = 
+    GtkAction *reload =
         gtk_ui_manager_get_action (m_UIManager, "/MenuBar/FileMenu/ReloadFile");
     gtk_action_set_sensitive (reload, sensitive);
 }
 
-void 
+void
 MainView::sensitiveRotateLeft (gboolean sensitive)
 {
-    GtkAction *rotateLeft = 
+    GtkAction *rotateLeft =
         gtk_ui_manager_get_action (m_UIManager, "/MenuBar/ViewMenu/RotateLeft");
     gtk_action_set_sensitive (rotateLeft, sensitive);
 }
@@ -502,7 +521,7 @@ MainView::sensitiveRotateLeft (gboolean sensitive)
 void
 MainView::sensitiveRotateRight (gboolean sensitive)
 {
-    GtkAction *rotateRight = gtk_ui_manager_get_action (m_UIManager, 
+    GtkAction *rotateRight = gtk_ui_manager_get_action (m_UIManager,
             "/MenuBar/ViewMenu/RotateRight");
     gtk_action_set_sensitive (rotateRight, sensitive);
 }
@@ -561,7 +580,7 @@ MainView::show (void)
                                  config.getWindowWidth (),
                                  config.getWindowHeight ());
     gtk_widget_show (m_MainWindow);
-    gtk_window_move (GTK_WINDOW (m_MainWindow), 
+    gtk_window_move (GTK_WINDOW (m_MainWindow),
                      config.getWindowX (), config.getWindowY ());
 }
 
@@ -591,8 +610,8 @@ MainView::showIndex (gboolean show)
     {
         gtk_widget_hide (m_Sidebar);
     }
-    GtkAction *showIndex = 
-        gtk_ui_manager_get_action (m_UIManager, 
+    GtkAction *showIndex =
+        gtk_ui_manager_get_action (m_UIManager,
                                    "/MenuBar/ViewMenu/ShowIndex");
     gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (showIndex), show);
 }
@@ -624,6 +643,45 @@ MainView::setCursor (ViewCursor cursorType)
     gdk_flush ();
 }
 
+void
+MainView::setFullScreen (gboolean fullScreen)
+{
+    GtkWidget *toolBar = gtk_ui_manager_get_widget (m_UIManager, "/ToolBar");
+    GtkWidget *menuBar = gtk_ui_manager_get_widget (m_UIManager, "/MenuBar");
+    if ( fullScreen )
+    {
+        gtk_window_fullscreen (GTK_WINDOW (m_MainWindow));
+        // Hide the menu bar, tool bar, status bar and the index bar. Then
+        // zoom to fit.
+        gtk_widget_hide (menuBar);
+        gtk_widget_hide (toolBar);
+        gtk_widget_hide (m_StatusBar);
+        gtk_widget_hide (m_Sidebar);
+        gtk_toggle_action_set_active (
+                GTK_TOGGLE_ACTION (gtk_ui_manager_get_action (m_UIManager,
+                                   "/MenuBar/ViewMenu/ZoomFit")), TRUE);
+    }
+    else
+    {
+        gtk_window_unfullscreen (GTK_WINDOW (m_MainWindow));
+        gtk_widget_show (menuBar);
+        // Show again the toolbar, status bar and index, only if it was
+        // enabled.
+        main_window_show_index_cb (
+                GTK_TOGGLE_ACTION (gtk_ui_manager_get_action (m_UIManager,
+                        "/MenuBar/ViewMenu/ShowIndex")),
+                (gpointer)m_Pter);
+        main_window_show_statusbar_cb (
+                GTK_TOGGLE_ACTION (gtk_ui_manager_get_action (m_UIManager,
+                        "/MenuBar/ViewMenu/ShowStatusBar")),
+                (gpointer)m_Pter);
+        main_window_show_toolbar_cb (
+                GTK_TOGGLE_ACTION (gtk_ui_manager_get_action (m_UIManager,
+                        "/MenuBar/ViewMenu/ShowToolBar")),
+                (gpointer)m_Pter);
+    }
+}
+
 void
 MainView::setNumberOfPagesText (const gchar *text)
 {
@@ -716,7 +774,7 @@ MainView::showStatusbar (gboolean show)
     GtkAction *toggleAction = gtk_ui_manager_get_action (m_UIManager,
                                             "/MenuBar/ViewMenu/ShowStatusBar");
     gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (toggleAction), show);
-    if ( show ) 
+    if ( show )
     {
         gtk_widget_show (m_StatusBar);
     }
@@ -1012,7 +1070,8 @@ main_window_about_box_cb (GtkWidget *widget, gpointer data)
     };
     gchar *licenseTranslated = g_strconcat (_(license[0]), "\n",
                                             _(license[1]), "\n",
-                                            _(license[2]), "\n", NULL);    
+                                            _(license[2]), "\n", NULL);
+    gtk_about_dialog_set_url_hook (main_window_about_box_url_hook, NULL, NULL);
     gtk_show_about_dialog (NULL,
             "name", _("ePDFView"),
             "version", VERSION,
@@ -1027,6 +1086,17 @@ main_window_about_box_cb (GtkWidget *widget, gpointer data)
     g_free (licenseTranslated);
 }
 
+
+void
+main_window_about_box_url_hook (GtkAboutDialog *about, const gchar *link,
+                                gpointer data)
+{
+    g_assert ( NULL != data && "The data parameter is NULL.");
+
+    MainPter *pter = (MainPter *)data;
+    pter->aboutBoxLinkActivated (link);
+}
+
 ///
 /// @brief The user tried to find a word in the document.
 ///
@@ -1039,6 +1109,18 @@ main_window_find_cb (GtkWidget *widget, gpointer data)
     pter->findActivated ();
 }
 
+///
+/// @brief The user tried to switch to full screen.
+///
+void
+main_window_fullscreen_cb (GtkToggleAction *action, gpointer data)
+{
+    g_assert ( NULL != data && "The data parameter is NULL.");
+
+    MainPter *pter = (MainPter *)data;
+    pter->fullScreenActivated (gtk_toggle_action_get_active (action));
+}
+
 ///
 /// @brief Called when the window is moved or resized.
 ///
