@@ -18,6 +18,7 @@
 #include <config.h>
 #include <gettext.h>
 #include <gtk/gtk.h>
+#include <gdk/gdkkeysyms.h>
 #include <epdfview.h>
 #include "PageView.h"
 
@@ -33,11 +34,11 @@ static gboolean page_view_button_release_cb (GtkWidget *, GdkEventButton *,
                                              gpointer);
 static gboolean page_view_mouse_motion_cb (GtkWidget *, GdkEventMotion *,
                                            gpointer);
-static void page_view_get_scrollbars_size (GtkWidget *widget, 
+static void page_view_get_scrollbars_size (GtkWidget *,
                                            gint *width, gint *height);
 static void page_view_resized_cb (GtkWidget *, GtkAllocation *, gpointer);
-static gboolean page_view_scrolled_cb (GtkWidget *widget, GdkEventScroll *,
-                                       gpointer);
+static gboolean page_view_scrolled_cb (GtkWidget *, GdkEventScroll *, gpointer);
+static gboolean page_view_keypress_cb (GtkWidget *, GdkEventKey *, gpointer);
 
 
 PageView::PageView ():
@@ -88,7 +89,7 @@ PageView::getSize (gint *width, gint *height)
 {
     g_assert (NULL != width && "Tried to save the width to a NULL pointer.");
     g_assert (NULL != height && "Tried to save the height to a NULL pointer.");
-  
+
     gint vScrollSize = 0;
     gint hScrollSize = 0;
     page_view_get_scrollbars_size (m_PageScroll, &vScrollSize, &hScrollSize);
@@ -117,7 +118,7 @@ PageView::makeRectangleVisible (DocumentRectangle &rect, gdouble scale)
     gdouble realX2 = rect.getX2 () * scale;
     gdouble docX1 = getHorizontalScroll () - PAGE_VIEW_PADDING;
     gdouble docX2 = docX1 + hAdjustment->page_size;
-    
+
     gdouble dx = 0.0;
     if ( realX1 < docX1 )
     {
@@ -132,7 +133,7 @@ PageView::makeRectangleVisible (DocumentRectangle &rect, gdouble scale)
             CLAMP (hAdjustment->value + dx,
                    hAdjustment->lower,
                    hAdjustment->upper - hAdjustment->page_size));
-    
+
     // Calculate the vertical adjustment.
     GtkAdjustment *vAdjustment = gtk_scrolled_window_get_vadjustment (
             GTK_SCROLLED_WINDOW (m_PageScroll));
@@ -209,13 +210,15 @@ void
 PageView::setPresenter (PagePter *pter)
 {
     IPageView::setPresenter (pter);
-    
+
     // When resizing.
     g_signal_connect (G_OBJECT (m_PageScroll), "size-allocate",
                       G_CALLBACK (page_view_resized_cb), pter);
     // When scrolling.
     g_signal_connect (G_OBJECT (m_PageScroll), "scroll-event",
                       G_CALLBACK (page_view_scrolled_cb), pter);
+    g_signal_connect (G_OBJECT (m_PageScroll), "key-press-event",
+                      G_CALLBACK (page_view_keypress_cb), pter);
 
     // And connect the motion, button press and release events.
     gtk_widget_add_events (m_EventBox, GDK_POINTER_MOTION_MASK |
@@ -241,7 +244,7 @@ PageView::scrollPage (gdouble scrollX, gdouble scrollY, gint dx, gint dy)
             CLAMP (scrollX - hAdjValue,
                    hAdjustment->lower,
                    hAdjustment->upper - hAdjustment->page_size));
-    
+
     GtkAdjustment *vAdjustment = gtk_scrolled_window_get_vadjustment (
             GTK_SCROLLED_WINDOW (m_PageScroll));
     gdouble vAdjValue = vAdjustment->page_size *
@@ -291,8 +294,8 @@ PageView::showText (const gchar *text)
     {
         gint width = gdk_pixbuf_get_width (originalPage);
         gint height = gdk_pixbuf_get_height (originalPage);
-       
-        PangoLayout *layout = 
+
+        PangoLayout *layout =
             gtk_widget_create_pango_layout (m_PageImage, text);
 
         // I want the text to be half the page's width.
@@ -314,7 +317,7 @@ PageView::showText (const gchar *text)
                                          (gint)realSize * PANGO_SCALE);        
         pango_layout_set_font_description (layout, fontDescription);
         pango_layout_get_pixel_extents (layout, NULL, &logicalRectangle);
-        
+
         // Once the font are set up, I just need to render 
         // the pixbuf to the pixmap (copying from client-side to 
         // server size) and draw the layout.
@@ -342,7 +345,7 @@ PageView::showText (const gchar *text)
         {
             g_object_unref (mask);
         }
-        
+
         gtk_image_set_from_pixbuf (GTK_IMAGE (m_PageImage), modifiedPage);
         g_object_unref (modifiedPage);
     } 
@@ -362,10 +365,24 @@ void
 PageView::getPagePosition (gint widgetX, gint widgetY, gint *pageX, gint *pageY)
 {
     g_assert ( NULL != pageX && "Tried to save the page's X to NULL.");
-    g_assert ( NULL != pageY && "Tried to save the page's Y to NULL.");    
+    g_assert ( NULL != pageY && "Tried to save the page's Y to NULL.");
 
-    *pageX = widgetX - PAGE_VIEW_PADDING + (gint)getHorizontalScroll ();
-    *pageY = widgetY - PAGE_VIEW_PADDING + (gint)getVerticalScroll ();
+    // Since the page is centered on the GtkImage widget, we need to
+    // get the current widget size and the current image size to know
+    // how many widget space is being used for padding.
+    gint horizontalPadding = PAGE_VIEW_PADDING;
+    gint verticalPadding = PAGE_VIEW_PADDING;
+    GdkPixbuf *page = gtk_image_get_pixbuf (GTK_IMAGE (m_PageImage));
+    if ( NULL != page )
+    {
+        horizontalPadding =
+            (m_PageImage->allocation.width - gdk_pixbuf_get_width (page)) / 2;
+        verticalPadding =
+            (m_PageImage->allocation.height - gdk_pixbuf_get_height (page)) / 2;
+    }
+
+    *pageX = widgetX - horizontalPadding + (gint)getHorizontalScroll ();
+    *pageY = widgetY - verticalPadding + (gint)getVerticalScroll ();
 }
 
 ///
@@ -408,6 +425,8 @@ page_view_button_press_cb (GtkWidget *widget, GdkEventButton *event,
     view->getPagePosition (event_x, event_y, &x, &y);
     view->getPresenter ()->mouseButtonPressed (event->button, x, y);
 
+    gtk_widget_grab_focus(view->getTopWidget());
+    
     return TRUE;
 }
 
@@ -435,7 +454,7 @@ page_view_mouse_motion_cb (GtkWidget *widget, GdkEventMotion *event,
 {
     g_assert ( NULL != data && "The data is NULL.");
     PageView *view = (PageView *)data;
-    
+
     gint event_x;
     gint event_y;
     gtk_widget_get_pointer (view->getTopWidget (), &event_x, &event_y);
@@ -453,28 +472,26 @@ page_view_get_scrollbars_size (GtkWidget *widget, gint *width, gint *height)
 {
     g_assert (NULL != width && "Tried to save the width to a NULL pointer.");
     g_assert (NULL != height && "Tried to save the height to a NULL pointer.");
-  
-    gint shadowWidth = 0;
-    gint shadowHeight = 0;
+
+    gint borderWidth = widget->style->xthickness;
+    gint borderHeight = widget->style->ythickness;
     if ( GTK_SHADOW_NONE !=
             gtk_scrolled_window_get_shadow_type (GTK_SCROLLED_WINDOW (widget)) )
     {
-        shadowWidth = widget->style->xthickness;
-        shadowHeight = widget->style->ythickness;
+        borderWidth += widget->style->xthickness;
+        borderHeight += widget->style->ythickness;
     }
-    
+
     gint scrollBarSpacing = 0;
     gtk_widget_style_get (widget, "scrollbar-spacing", &scrollBarSpacing, NULL);
-    
-    GtkWidget *vScrollBar = 
-        gtk_scrolled_window_get_vscrollbar (GTK_SCROLLED_WINDOW (widget));
+
+    GtkWidget *vScrollBar = GTK_SCROLLED_WINDOW (widget)->vscrollbar;
     *width = vScrollBar->allocation.width +
-             (scrollBarSpacing + PAGE_VIEW_PADDING + shadowWidth) * 2;
-    
-    GtkWidget *hScrollBar =
-        gtk_scrolled_window_get_hscrollbar (GTK_SCROLLED_WINDOW (widget));
+             (PAGE_VIEW_PADDING + borderWidth) * 2 + scrollBarSpacing;
+
+    GtkWidget *hScrollBar = GTK_SCROLLED_WINDOW (widget)->hscrollbar;
     *height = hScrollBar->allocation.height +
-              (scrollBarSpacing + PAGE_VIEW_PADDING + shadowHeight) * 2;
+              (PAGE_VIEW_PADDING + borderHeight) * 2 + scrollBarSpacing;
 }
 
 ///
@@ -485,7 +502,7 @@ page_view_resized_cb (GtkWidget *widget, GtkAllocation *allocation,
                                   gpointer data)
 {
     g_assert ( NULL != data && "The data parameter is NULL.");
-    
+
     gint vScrollSize = 0;
     gint hScrollSize = 0;
     page_view_get_scrollbars_size (widget, &vScrollSize, &hScrollSize);
@@ -525,3 +542,77 @@ page_view_scrolled_cb (GtkWidget *widget, GdkEventScroll *event, gpointer data)
 
     return FALSE;
 }
+
+///
+/// @brief A key was pressed.
+///
+static gboolean
+page_view_keypress_cb(GtkWidget *widget, GdkEventKey *event, gpointer data)
+{
+    g_assert ( NULL != data && "The data parameter is NULL.");
+
+    GtkScrollType direction;
+    gboolean horizontal = FALSE;
+    PagePter *pter = (PagePter *)data;
+
+    GtkAdjustment *adjustment = 
+        gtk_scrolled_window_get_vadjustment (GTK_SCROLLED_WINDOW (widget));
+    gdouble position = gtk_adjustment_get_value (adjustment);
+
+    if ( event->state & (GDK_SHIFT_MASK | GDK_CONTROL_MASK) )
+    {
+        return FALSE;
+    }
+
+    switch ( event->keyval )
+    {
+        case GDK_Left:
+        case GDK_KP_Left:
+            direction = GTK_SCROLL_STEP_LEFT;
+            horizontal = TRUE;
+            break;
+
+        case GDK_Right:
+        case GDK_KP_Right:
+            horizontal = TRUE;
+            direction = GTK_SCROLL_STEP_RIGHT;
+            break;
+
+        case GDK_Up:
+        case GDK_KP_Up:
+            direction = GTK_SCROLL_STEP_UP;
+            break;
+
+        case GDK_Down:
+        case GDK_KP_Down:
+            direction = GTK_SCROLL_STEP_DOWN;
+            break;
+
+        case GDK_Page_Up:
+        case GDK_KP_Page_Up:
+            if ( position == adjustment->lower )
+            {
+                pter->scrollToPreviousPage ();
+                return TRUE;
+            }
+            direction = GTK_SCROLL_START;
+            break;
+
+        case GDK_Page_Down:
+        case GDK_KP_Page_Down:
+            if ( position == ( adjustment->upper - adjustment->page_size) )
+            {
+                pter->scrollToNextPage ();
+                return TRUE;
+            }
+            direction = GTK_SCROLL_END;
+            break;
+
+        default:
+            return FALSE;
+    }
+
+    g_signal_emit_by_name(G_OBJECT(widget), "scroll-child",
+                          direction, horizontal);
+    return TRUE;
+}
