@@ -73,7 +73,7 @@ void
 MainPterTest::initialStatus ()
 {
     CPPUNIT_ASSERT ( m_View->isShown () );
-    CPPUNIT_ASSERT_EQUAL (0, 
+    CPPUNIT_ASSERT_EQUAL (0,
             g_ascii_strcasecmp ("PDF Viewer", m_View->getTitle ()));
     CPPUNIT_ASSERT (!m_View->isSensitiveGoToFirstPage ());
     CPPUNIT_ASSERT (!m_View->isSensitiveGoToLastPage ());
@@ -83,6 +83,8 @@ MainPterTest::initialStatus ()
     CPPUNIT_ASSERT (!m_View->isSensitiveReload ());
     CPPUNIT_ASSERT (!m_View->isSensitiveRotateLeft ());
     CPPUNIT_ASSERT (!m_View->isSensitiveRotateRight ());
+    CPPUNIT_ASSERT (!m_View->isSensitiveSave ());
+    CPPUNIT_ASSERT (!m_View->isSensitiveZoom ());
     CPPUNIT_ASSERT (!m_View->isSensitiveZoomIn ());
     CPPUNIT_ASSERT (!m_View->isSensitiveZoomOut ());
     CPPUNIT_ASSERT (!m_View->isSensitiveZoomFit ());
@@ -90,6 +92,9 @@ MainPterTest::initialStatus ()
     CPPUNIT_ASSERT (!m_View->isShownIndex ());
     CPPUNIT_ASSERT (m_View->isShownToolbar ());
     CPPUNIT_ASSERT (m_View->isShownStatusbar ());
+#if defined (HAVE_CUPS)
+    CPPUNIT_ASSERT (!m_View->isSensitivePrint ());
+#endif // HAVE_CUPS
 }
 
 ///
@@ -107,7 +112,7 @@ MainPterTest::loadDocument ()
     m_View->setOpenFileName ("/tmp/test.pdf");
     m_MainPter->openFileActivated ();
     m_MainPter->waitForFileLoaded ();
-    CPPUNIT_ASSERT_EQUAL (0, 
+    CPPUNIT_ASSERT_EQUAL (0,
             g_ascii_strcasecmp ("/tmp/test.pdf", m_View->getTitle ()));
     CPPUNIT_ASSERT (!m_View->isSensitiveGoToFirstPage ());
     CPPUNIT_ASSERT (m_View->isSensitiveGoToLastPage ());
@@ -117,11 +122,16 @@ MainPterTest::loadDocument ()
     CPPUNIT_ASSERT (m_View->isSensitiveReload ());
     CPPUNIT_ASSERT (m_View->isSensitiveRotateLeft ());
     CPPUNIT_ASSERT (m_View->isSensitiveRotateRight ());
+    CPPUNIT_ASSERT (m_View->isSensitiveSave ());
+    CPPUNIT_ASSERT (m_View->isSensitiveZoom ());
     CPPUNIT_ASSERT (m_View->isSensitiveZoomIn ());
     CPPUNIT_ASSERT (m_View->isSensitiveZoomOut ());
     CPPUNIT_ASSERT (m_View->isSensitiveZoomFit ());
     CPPUNIT_ASSERT (m_View->isSensitiveZoomWidth ());
     CPPUNIT_ASSERT (!m_View->isShownIndex ());
+#if defined (HAVE_CUPS)
+    CPPUNIT_ASSERT (m_View->isSensitivePrint ());
+#endif // HAVE_CUPS
 
     // Now try a document with a title.
     m_Document->setTitle (g_strdup ("Test PDF"));
@@ -137,11 +147,16 @@ MainPterTest::loadDocument ()
     CPPUNIT_ASSERT (m_View->isSensitiveReload ());
     CPPUNIT_ASSERT (m_View->isSensitiveRotateLeft ());
     CPPUNIT_ASSERT (m_View->isSensitiveRotateRight ());
+    CPPUNIT_ASSERT (m_View->isSensitiveSave ());
+    CPPUNIT_ASSERT (m_View->isSensitiveZoom ());
     CPPUNIT_ASSERT (m_View->isSensitiveZoomIn ());
     CPPUNIT_ASSERT (m_View->isSensitiveZoomOut ());
     CPPUNIT_ASSERT (m_View->isSensitiveZoomFit ());
     CPPUNIT_ASSERT (m_View->isSensitiveZoomWidth ());
     CPPUNIT_ASSERT (!m_View->isShownIndex ());
+#if defined (HAVE_CUPS)
+    CPPUNIT_ASSERT (m_View->isSensitivePrint ());
+#endif // HAVE_CUPS
 }
 
 ///
@@ -155,7 +170,7 @@ MainPterTest::loadCanceled ()
     // returning a NULL file name is what will happen when cancel.
     m_View->setOpenFileName (NULL);
     m_MainPter->openFileActivated ();
-    CPPUNIT_ASSERT_EQUAL (0, 
+    CPPUNIT_ASSERT_EQUAL (0,
             g_ascii_strcasecmp ("PDF Viewer", m_View->getTitle ()));
     CPPUNIT_ASSERT (!m_View->isSensitiveGoToFirstPage ());
     CPPUNIT_ASSERT (!m_View->isSensitiveGoToLastPage ());
@@ -165,11 +180,16 @@ MainPterTest::loadCanceled ()
     CPPUNIT_ASSERT (!m_View->isSensitiveReload ());
     CPPUNIT_ASSERT (!m_View->isSensitiveRotateLeft ());
     CPPUNIT_ASSERT (!m_View->isSensitiveRotateRight ());
+    CPPUNIT_ASSERT (!m_View->isSensitiveSave ());
+    CPPUNIT_ASSERT (!m_View->isSensitiveZoom ());
     CPPUNIT_ASSERT (!m_View->isSensitiveZoomIn ());
     CPPUNIT_ASSERT (!m_View->isSensitiveZoomOut ());
     CPPUNIT_ASSERT (!m_View->isSensitiveZoomFit ());
     CPPUNIT_ASSERT (!m_View->isSensitiveZoomWidth ());
     CPPUNIT_ASSERT (!m_View->shownError ());
+#if defined (HAVE_CUPS)
+    CPPUNIT_ASSERT (!m_View->isSensitivePrint ());
+#endif // HAVE_CUPS
 }
 
 
@@ -182,12 +202,12 @@ MainPterTest::loadCanceled ()
 ///
 void
 MainPterTest::loadFailed ()
-{    
+{
     m_View->setOpenFileName ("/tmp/test.pdf");
     m_Document->setOpenError (DocumentErrorDamaged);
     m_MainPter->openFileActivated ();
     m_MainPter->waitForFileLoaded ();
-    CPPUNIT_ASSERT_EQUAL (0, 
+    CPPUNIT_ASSERT_EQUAL (0,
             g_ascii_strcasecmp ("PDF Viewer", m_View->getTitle ()));
     CPPUNIT_ASSERT (!m_View->isSensitiveGoToFirstPage ());
     CPPUNIT_ASSERT (!m_View->isSensitiveGoToLastPage ());
@@ -197,11 +217,16 @@ MainPterTest::loadFailed ()
     CPPUNIT_ASSERT (!m_View->isSensitiveReload ());
     CPPUNIT_ASSERT (!m_View->isSensitiveRotateLeft ());
     CPPUNIT_ASSERT (!m_View->isSensitiveRotateRight ());
+    CPPUNIT_ASSERT (!m_View->isSensitiveSave ());
+    CPPUNIT_ASSERT (!m_View->isSensitiveZoom ());
     CPPUNIT_ASSERT (!m_View->isSensitiveZoomIn ());
     CPPUNIT_ASSERT (!m_View->isSensitiveZoomOut ());
     CPPUNIT_ASSERT (!m_View->isSensitiveZoomFit ());
     CPPUNIT_ASSERT (!m_View->isSensitiveZoomWidth ());
     CPPUNIT_ASSERT (m_View->shownError ());
+#if defined (HAVE_CUPS)
+    CPPUNIT_ASSERT (!m_View->isSensitivePrint ());
+#endif // HAVE_CUPS
 }
 
 ///
@@ -220,7 +245,7 @@ MainPterTest::canceledPassword ()
     m_Document->setOpenError (DocumentErrorEncrypted);
     m_MainPter->openFileActivated ();
     m_MainPter->waitForFileLoaded ();
-    CPPUNIT_ASSERT_EQUAL (0, 
+    CPPUNIT_ASSERT_EQUAL (0,
             g_ascii_strcasecmp ("PDF Viewer", m_View->getTitle ()));
     CPPUNIT_ASSERT (!m_View->isSensitiveGoToFirstPage ());
     CPPUNIT_ASSERT (!m_View->isSensitiveGoToLastPage ());
@@ -230,11 +255,16 @@ MainPterTest::canceledPassword ()
     CPPUNIT_ASSERT (!m_View->isSensitiveReload ());
     CPPUNIT_ASSERT (!m_View->isSensitiveRotateLeft ());
     CPPUNIT_ASSERT (!m_View->isSensitiveRotateRight ());
+    CPPUNIT_ASSERT (!m_View->isSensitiveSave ());
+    CPPUNIT_ASSERT (!m_View->isSensitiveZoom ());
     CPPUNIT_ASSERT (!m_View->isSensitiveZoomIn ());
     CPPUNIT_ASSERT (!m_View->isSensitiveZoomOut ());
     CPPUNIT_ASSERT (!m_View->isSensitiveZoomFit ());
     CPPUNIT_ASSERT (!m_View->isSensitiveZoomWidth ());
     CPPUNIT_ASSERT (!m_View->shownError ());
+#if defined (HAVE_CUPS)
+    CPPUNIT_ASSERT (!m_View->isSensitivePrint ());
+#endif // HAVE_CUPS
 }
 
 ///
@@ -253,7 +283,7 @@ MainPterTest::badPassword ()
     m_Document->setOpenError (DocumentErrorEncrypted);
     m_MainPter->openFileActivated ();
     m_MainPter->waitForFileLoaded ();
-    CPPUNIT_ASSERT_EQUAL (0, 
+    CPPUNIT_ASSERT_EQUAL (0,
             g_ascii_strcasecmp ("PDF Viewer", m_View->getTitle ()));
     CPPUNIT_ASSERT (!m_View->isSensitiveGoToFirstPage ());
     CPPUNIT_ASSERT (!m_View->isSensitiveGoToLastPage ());
@@ -263,12 +293,17 @@ MainPterTest::badPassword ()
     CPPUNIT_ASSERT (!m_View->isSensitiveReload ());
     CPPUNIT_ASSERT (!m_View->isSensitiveRotateLeft ());
     CPPUNIT_ASSERT (!m_View->isSensitiveRotateRight ());
+    CPPUNIT_ASSERT (!m_View->isSensitiveSave ());
+    CPPUNIT_ASSERT (!m_View->isSensitiveZoom ());
     CPPUNIT_ASSERT (!m_View->isSensitiveZoomIn ());
     CPPUNIT_ASSERT (!m_View->isSensitiveZoomOut ());
     CPPUNIT_ASSERT (!m_View->isSensitiveZoomFit ());
     CPPUNIT_ASSERT (!m_View->isSensitiveZoomWidth ());
     CPPUNIT_ASSERT (m_View->shownError ());
     CPPUNIT_ASSERT_EQUAL (3, m_View->countTimesShownPasswordPrompt ());
+#if defined (HAVE_CUPS)
+    CPPUNIT_ASSERT (!m_View->isSensitivePrint ());
+#endif // HAVE_CUPS
 }
 
 ///
@@ -286,7 +321,7 @@ MainPterTest::goodPassword ()
     m_Document->setOpenError (DocumentErrorEncrypted);
     m_MainPter->openFileActivated ();
     m_MainPter->waitForFileLoaded ();
-    CPPUNIT_ASSERT_EQUAL (0, 
+    CPPUNIT_ASSERT_EQUAL (0,
             g_ascii_strcasecmp ("/tmp/test.pdf", m_View->getTitle ()));
     CPPUNIT_ASSERT (!m_View->isSensitiveGoToFirstPage ());
     CPPUNIT_ASSERT (m_View->isSensitiveGoToLastPage ());
@@ -296,12 +331,17 @@ MainPterTest::goodPassword ()
     CPPUNIT_ASSERT (m_View->isSensitiveReload ());
     CPPUNIT_ASSERT (m_View->isSensitiveRotateLeft ());
     CPPUNIT_ASSERT (m_View->isSensitiveRotateRight ());
+    CPPUNIT_ASSERT (m_View->isSensitiveSave ());
+    CPPUNIT_ASSERT (m_View->isSensitiveZoom ());
     CPPUNIT_ASSERT (m_View->isSensitiveZoomIn ());
     CPPUNIT_ASSERT (m_View->isSensitiveZoomOut ());
     CPPUNIT_ASSERT (m_View->isSensitiveZoomFit ());
     CPPUNIT_ASSERT (m_View->isSensitiveZoomWidth ());
     CPPUNIT_ASSERT (!m_View->shownError ());
     CPPUNIT_ASSERT_EQUAL (1, m_View->countTimesShownPasswordPrompt ());
+#if defined (HAVE_CUPS)
+    CPPUNIT_ASSERT (m_View->isSensitivePrint ());
+#endif // HAVE_CUPS
 }
 
 ///
@@ -312,7 +352,7 @@ MainPterTest::goodPassword ()
 /// view can show it up when opening the dialog.
 ///
 void
-MainPterTest::lastFolder ()
+MainPterTest::openLastFolder ()
 {
     m_View->setOpenFileName ("/tmp/test.pdf");
     m_MainPter->openFileActivated ();
@@ -325,13 +365,81 @@ MainPterTest::lastFolder ()
     m_MainPter->waitForFileLoaded ();
     CPPUNIT_ASSERT (0 == g_ascii_strcasecmp ("/tmp", 
                                              m_View->getLastOpenFileFolder ()));
-    
+
     m_MainPter->openFileActivated ();
     m_MainPter->waitForFileLoaded ();
     CPPUNIT_ASSERT (0 == g_ascii_strcasecmp ("/usr", 
                                              m_View->getLastOpenFileFolder ()));
 }
 
+///
+/// @brief Test saving a document.
+///
+/// Saving a document will call the saveFile() function of the document's class.
+///
+void
+MainPterTest::saveDocument ()
+{
+    m_View->setOpenFileName ("/tmp/test.pdf");
+    m_MainPter->openFileActivated ();
+    m_MainPter->waitForFileLoaded ();
+    m_View->setSaveFileName ("/tmp/copy.pdf");
+    m_MainPter->saveFileActivated ();
+    CPPUNIT_ASSERT (0 == g_ascii_strcasecmp ("/tmp/copy.pdf",
+                                             m_Document->getSavedFileName ()));
+}
+
+///
+/// @brief Tests a canceled save file.
+///
+/// When the user cancels the save dialog, then nothing should be saved.
+///
+void
+MainPterTest::saveCanceled ()
+{
+    m_View->setOpenFileName ("/tmp/test.pdf");
+    m_MainPter->openFileActivated ();
+    m_MainPter->waitForFileLoaded ();
+    // returning a NULL file name is what will happen when cancel.
+    m_View->setSaveFileName (NULL);
+    m_MainPter->saveFileActivated ();
+    CPPUNIT_ASSERT (0 == g_ascii_strcasecmp ("",
+                                             m_Document->getSavedFileName ()));
+}
+
+///
+/// @brief Test the last folder used to save a file.
+///
+/// When the presenter wants to save a file, it should pass to the
+/// view the last folder that was used to save a file, so the
+/// view can show it up when saving the dialog.
+///
+void
+MainPterTest::saveLastFolder ()
+{
+    m_View->setOpenFileName ("/tmp/test.pdf");
+    m_MainPter->openFileActivated ();
+    m_MainPter->waitForFileLoaded ();
+    m_View->setSaveFileName ("/tmp/copy.pdf");
+    m_MainPter->saveFileActivated ();
+    m_MainPter->waitForFileSaved ();
+    CPPUNIT_ASSERT (NULL == m_View->getLastSaveFileFolder ());
+
+
+    m_View->setSaveFileName ("/usr/copy.pdf");
+    m_MainPter->saveFileActivated ();
+    m_MainPter->waitForFileSaved ();
+    CPPUNIT_ASSERT (0 == g_ascii_strcasecmp ("/tmp",
+                                             m_View->getLastSaveFileFolder ()));
+
+    m_MainPter->saveFileActivated ();
+    m_MainPter->waitForFileSaved ();
+    CPPUNIT_ASSERT (0 == g_ascii_strcasecmp ("/usr",
+                                             m_View->getLastSaveFileFolder ()));
+
+}
+
+
 ///
 /// @brief Test page navigation.
 ///
@@ -345,8 +453,7 @@ MainPterTest::pageNavigation ()
     m_Document->setNumPages (4);
     m_MainPter->openFileActivated ();
     m_MainPter->waitForFileLoaded ();
-    // Check that sets the correct number of pages and the current page.
-    CPPUNIT_ASSERT_EQUAL (4, m_View->getTotalPages ());
+    // Check that sets the correct current page.
     CPPUNIT_ASSERT_EQUAL (1, m_View->getCurrentPage ());
     // Going to the next page should make all actions sensitive.
     m_MainPter->goToNextPageActivated ();
@@ -414,8 +521,7 @@ MainPterTest::pageNavigationEntry ()
     m_Document->setNumPages (4);
     m_MainPter->openFileActivated ();
     m_MainPter->waitForFileLoaded ();
-    // Check that sets the correct number of pages and the current page.
-    CPPUNIT_ASSERT_EQUAL (4, m_View->getTotalPages ());
+    // Check that sets the correct current page.
     CPPUNIT_ASSERT_EQUAL (1, m_View->getCurrentPage ());
 
     // Now try again using the page entry on toolbar.
@@ -517,7 +623,7 @@ MainPterTest::pageZoomInAndOut ()
 
     m_MainPter->zoomOutActivated ();
     CPPUNIT_ASSERT_DOUBLES_EQUAL (1.0, m_Document->getZoom (), 0.0001);
-   
+
     CPPUNIT_ASSERT (m_View->isSensitiveZoomIn ());
     CPPUNIT_ASSERT (m_View->isSensitiveZoomOut ());
     // Make sure we go to the last zoom level.
@@ -544,7 +650,7 @@ MainPterTest::pageZoomWidth ()
     m_View->setOpenFileName ("/tmp/test.pdf");
     m_MainPter->openFileActivated ();
     m_MainPter->waitForFileLoaded ();
-    
+
     m_MainPter->zoomWidthActivated (TRUE);
     CPPUNIT_ASSERT (m_View->isZoomToWidthActive ());
     m_MainPter->zoomWidthActivated (FALSE);
@@ -569,7 +675,7 @@ MainPterTest::pageZoomFit ()
     m_View->setOpenFileName ("/tmp/test.pdf");
     m_MainPter->openFileActivated ();
     m_MainPter->waitForFileLoaded ();
-    
+
     m_MainPter->zoomFitActivated (TRUE);
     CPPUNIT_ASSERT (m_View->isZoomToFitActive ());
     m_MainPter->zoomFitActivated (FALSE);
@@ -606,9 +712,9 @@ MainPterTest::pageZoomAndRotate ()
     // Rotate left. 75/250 = 0.3
     m_MainPter->rotateLeftActivated ();
     CPPUNIT_ASSERT_DOUBLES_EQUAL (0.3, m_Document->getZoom (), 0.0001);
-    m_MainPter->rotateRightActivated ();    
+    m_MainPter->rotateRightActivated ();
     CPPUNIT_ASSERT_DOUBLES_EQUAL (0.75, m_Document->getZoom (), 0.0001);
-    m_MainPter->rotateRightActivated ();    
+    m_MainPter->rotateRightActivated ();
     CPPUNIT_ASSERT_DOUBLES_EQUAL (0.3, m_Document->getZoom (), 0.0001);
     m_MainPter->rotateLeftActivated ();
     CPPUNIT_ASSERT_DOUBLES_EQUAL (0.75, m_Document->getZoom (), 0.0001);
@@ -620,14 +726,58 @@ MainPterTest::pageZoomAndRotate ()
     // Rotate left, 75/250 = 0.3
     m_MainPter->rotateLeftActivated ();
     CPPUNIT_ASSERT_DOUBLES_EQUAL (0.3, m_Document->getZoom (), 0.0001);
-    m_MainPter->rotateRightActivated ();    
+    m_MainPter->rotateRightActivated ();
     CPPUNIT_ASSERT_DOUBLES_EQUAL (0.2, m_Document->getZoom (), 0.0001);
-    m_MainPter->rotateRightActivated ();    
+    m_MainPter->rotateRightActivated ();
     CPPUNIT_ASSERT_DOUBLES_EQUAL (0.3, m_Document->getZoom (), 0.0001);
     m_MainPter->rotateLeftActivated ();
     CPPUNIT_ASSERT_DOUBLES_EQUAL (0.2, m_Document->getZoom (), 0.0001);
 }
 
+///
+/// @brief Test the current zoom level indicator.
+///
+/// The main view has an entry in the tool bar that tells the user
+/// which is the current document's zoom level. Also, when the user
+/// enter a new zoom level and presses ENTER, the new zoom level is
+/// used.
+///
+void
+MainPterTest::pageZoomIndicator ()
+{
+    // When the open is first opened, the zoom is 100%.
+    m_View->setOpenFileName ("/tmp/test.pdf");
+    m_MainPter->openFileActivated ();
+    m_MainPter->waitForFileLoaded ();
+    CPPUNIT_ASSERT (0 == g_ascii_strcasecmp ("100%", m_View->getZoomText ()));
+
+    // Try changing a little the zoom level.
+    m_MainPter->zoomInActivated ();
+    CPPUNIT_ASSERT (0 == g_ascii_strcasecmp ("120%", m_View->getZoomText ()));
+    m_MainPter->zoomInActivated ();
+    CPPUNIT_ASSERT (0 == g_ascii_strcasecmp ("144%", m_View->getZoomText ()));
+    m_MainPter->zoomOutActivated ();
+    CPPUNIT_ASSERT (0 == g_ascii_strcasecmp ("120%", m_View->getZoomText ()));
+    m_MainPter->zoomOutActivated ();
+    CPPUNIT_ASSERT (0 == g_ascii_strcasecmp ("100%", m_View->getZoomText ()));
+    m_MainPter->zoomOutActivated ();
+    CPPUNIT_ASSERT (0 == g_ascii_strcasecmp ("83.3%", m_View->getZoomText ()));
+
+    // Now try to set the zoom level by hand. If the zoom level is 
+    // clamped 400% and 5.41%.
+    m_View->setZoomText ("50%");
+    m_MainPter->zoomActivated ();
+    CPPUNIT_ASSERT_DOUBLES_EQUAL (0.5, m_Document->getZoom (), 0.0001);
+    m_View->setZoomText ("1000%");
+    m_MainPter->zoomActivated ();
+    CPPUNIT_ASSERT_DOUBLES_EQUAL (4.0, m_Document->getZoom (), 0.0001);
+    CPPUNIT_ASSERT (0 == g_ascii_strcasecmp ("400%", m_View->getZoomText ()));
+    m_View->setZoomText ("1");
+    m_MainPter->zoomActivated ();
+    CPPUNIT_ASSERT_DOUBLES_EQUAL (0.0541, m_Document->getZoom (), 0.0001);
+    CPPUNIT_ASSERT (0 == g_ascii_strcasecmp ("5.41%", m_View->getZoomText ()));
+}
+
 ///
 /// @brief Test to reload a normal document.
 ///
